package adaptools.utils;
// -----------------------------------------------------------------------------
// ListFilesSubs.java
// -----------------------------------------------------------------------------

/*
 * =============================================================================
 * Copyright (c) 1998-2005 Jeffrey M. Hunter. All rights reserved.
 * 
 * All source code and material located at the Internet address of
 * http://www.idevelopment.info is the copyright of Jeffrey M. Hunter, 2005 and
 * is protected under copyright laws of the United States. This source code may
 * not be hosted on any other site without my express, prior, written
 * permission. Application to host any of the material elsewhere can be made by
 * contacting me at jhunter@idevelopment.info.
 *
 * I have made every effort and taken great care in making sure that the source
 * code and other content included on my web site is technically accurate, but I
 * disclaim any and all responsibility for any loss, damage or destruction of
 * data or any other property which may arise from relying on it. I will in no
 * case be liable for any monetary damages arising from such loss, damage or
 * destruction.
 * 
 * As with any code, ensure to test this code in a development environment 
 * before attempting to run it in production.
 * =============================================================================
 */
 
import java.io.File;
import java.io.FileFilter;
import java.io.FilenameFilter;
import java.net.URI;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * -----------------------------------------------------------------------------
 * This program demonstrates how to list the file or subdirectories in a
 * given directory
 * 
 * @version 1.0
 * @author  Jeffrey M. Hunter  (jhunter@idevelopment.info)
 * @author  http://www.idevelopment.info
 * -----------------------------------------------------------------------------
 */

public class ListFilesSubs {

    public static String doGetDir() {
    	
    	//File dir = new File(System.getenv("CAMINHOADP"));
    	File dir = new File(System.getProperty("user.dir"));
    	URI uri = dir.toURI();
    	return uri.getPath();    	
        //return System.getProperty("user.dir");      
    }


    /**
     * Used to list the files / subdirectories in a given directory and also
     * provide a filter class.
     * @param dir Directory to start listing from
     * @param ff  A string that can be used to filter out files from the
     *            returned list of files. In this example, the String
     *            values is used to only return those values that start
     *            with the given String.
     */
    public static File[] doFileFilterListing(String dirName, String regex) 
    {
    
        final String fileFilter = regex;

        File dir = new File(dirName);
        FilenameFilter filter  = null;
        
        if (fileFilter != null) {        	
        		
	        	// It is also possible to filter the list of returned files.
	            // This example uses the passed in String value (if any) to only
	            // list those files that start with the given String.
	        	filter = new FilenameFilter() {
	                public boolean accept(File dir, String name) {
	                	Pattern padrao;
	                	Matcher pesquisa;
	                	
	                	padrao = Pattern.compile(fileFilter);
	                	pesquisa = padrao.matcher(name);
	                	                	
	                	return pesquisa.matches();        
	                }
	            };
        }

        return dir.listFiles(filter);
    }


    /**
     * Used to list the files / subdirectories in a given directory and also
     * provide a filter class that only lists the directories.
     * @param dir Directory to start listing from
     */
    public static File[] doFileFilterDirectoryListing(String dirName) {

    	File dir = new File(dirName);
        
        // The list of files can also be retrieved as File objects. In this
        // case, we are just listing all files in the directory. For the sake
        // of this example, we will not print them out here.
        //File[] files = (new File(dirName)).listFiles();

        // This filter only returns directories
        FileFilter dirFilter = new FileFilter() {
            public boolean accept(File dir) {
                return dir.isDirectory();
            }
        };

        return dir.listFiles(dirFilter);
    }
    
    /**
     * Used to list the files / subdirectories in a given directory and also
     * provide a filter class that only lists the directories.
     * @param dir Directory to start listing from
     */
    public static File[] doFileFilterFileListing(String dirName) {

        File dir = new File(dirName);

        // The list of files can also be retrieved as File objects. In this
        // case, we are just listing all files in the directory. For the sake
        // of this example, we will not print them out here.
        //File[] files = (new File(dirName)).listFiles();

        // This filter only returns directories
        FileFilter dirFilter = new FileFilter() {
            public boolean accept(File dir) {
                return dir.isFile();
            }
        };

        return dir.listFiles(dirFilter);
    }

    /**
     * Sole entry point to the class and application.
     * @param args Array of String arguments.
     */
    public static void main(String[] args) {

        File[] files;

        /*
         * Lista todos os arquivos de um diretrio
         * se o parametro regex for passado como null, retorna todos os
         * arquivos e subdiretrios. 
         */
        files = doFileFilterListing("/home/leandro/temp", ".*[.]rar");
        
        for (int i=0; i<files.length; i++) {
            System.out.println(files[i]);
        }
        System.out.println();
        
        
        /*
         * Lista todos os subdiretrios de um determinado diretrio 
         */
        files = doFileFilterDirectoryListing("/home/leandro/temp");

        for (int i=0; i<files.length; i++) {
            System.out.println("[D] : " + files[i]);
        }
        System.out.println();
        
        
        /*
         * Lista todos os arquivos de um determinado diretrio
         */
        files = doFileFilterFileListing("/home/leandro/temp");
        

        for (int i=0; i<files.length; i++) {
            System.out.println("[F] : " + files[i]);            
        }
        
        System.out.println();
        
        
        /*
         * Apresenta o diretrio corrente
         */
        System.out.println("The current working directory is:\n " + doGetDir());
    }
}
