
/** VmdsImpl.java

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 @author Hemerson Pistori (pistori@ec.ucdb.br)

 This work is based on Dr. Joao Jose Neto theories and techniques on
 Automata and Formal Languages (http://www.pcs.usp.br/~lta).

*/
package adaptools.vm;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Stack;
import java.util.StringTokenizer;
import java.util.Vector;

/**
*
*   An implementation of Vmds that is to be used in non-visual
*   execution of an automaton.
*
*   @author Hemerson Pistori (pistori@ec.ucdb.br)
*
*/

public class VmdsImpl implements Vmds {
	
    private Vmds inputMachine = null;
    private Vmds outputMachine = null;

    private Kernel kernel = null;

    private int stepSpeed = MAX_SPEED;

    private int debugLevel = 1;

    private String currentDir = "";
    private String objectFileName = "";
    private String semanticAction = "";
    private String inputFileName = "";
    private String transducerOutputFileName = "";
    private String outputFileName = "";

	// Virtual Machine Data Structures:
	private Rules code = null;
	private int currentState = 0;
    StringBuffer inputBuffer = new StringBuffer();
    private int posToken = 0;
    StringBuffer outputBuffer = new StringBuffer();
    StringBuffer transducerBuffer = new StringBuffer();
    Stack stack;
    Vector finals;

    private Vmds caller = null;

    /**
     * Construtor padro
     * 
     * Cria um objeto denominado <h3>code</h3> do tipo 
     * <code>Rules</code>
     *
     */
    public VmdsImpl() {
        code = new Rules(MAX_OBJECT_SIZE, Rules.TOTAL_COLUMNS);
	}

    public void init(String fileName) {
    }

    public int getCurState() {
		return currentState;
    }

    public Token curToken() {
		Token token;
		if(inputMachine == null) {  // USE DEFAULT LEXER.
    		String value = inputBuffer.toString();
		    if( value == null || value.length() < 1 ) {
			   token = null;
	        } else {


			   if(posToken>=value.length()) token = null;
			   else {
//     			   if( value.charAt(posToken) == ' ' ||
//				       value.charAt(posToken) == '\n' ||
//				       value.charAt(posToken) == '\r' ||
//				       value.charAt(posToken) == '\t'
//				       )
//				       token = (new Token("spc",value.substring(posToken,posToken+1)));
//				   else
			           token = (new Token(value.substring(posToken,posToken+1),value.substring(posToken,posToken+1)));
			   }
	        }
	     }
		 else {
			 token = getLastToken();
	     }

  //System.out.println(objectFileName+" curTok = "+token.label);

    	 return(token);

    }

    /** method called by {@link curToken} when there is an input conexion */
    private Token getLastToken() {
         Kernel inputKernel = inputMachine.getKernel();
         if(inputKernel == null) System.out.println("Threading Problems ... Input Kernel Not Loaded");
		 if(inputMachine.getKernel() == null || inputMachine.getKernel().isHasToken() == false) {
//			 System.out.println("Vai aguardar token");
			 kernel.waitToken();
	     }
		 return(inputKernel.getLastToken());
    }

    /**
     * read the Current Token
     */
    public void readToken() {
		if(inputMachine == null) {  // USE DEFAULT LEXER.
    		String value = inputBuffer.toString();
    		if(posToken < value.length()) {
			    if(debugLevel > 0) System.out.print(value.substring(posToken,posToken+1));
				++posToken;
		    }
	    } else {
			inputMachine.getKernel().tokenRead();
	    }
    	showCurrentToken();
    }


    public void appendOutput(String value) {
	    outputBuffer.append(value);
    }

    // Append text to the Transducer Output Window.
    public void appendTransOutput(String value) {
	    transducerBuffer.append(value);
    }


    // Initialize Stack and other auxiliary data structures.
    public void initDS() {
		stack = new Stack();
		posToken = 0;
	    outputBuffer = new StringBuffer();
	    transducerBuffer = new StringBuffer();
    }

    public void setCurState(int state) {
		currentState = state;
    }

    public int pop() {
		if(stack.empty()) return( STACK_UNDERFLOW );
		int value = ((Integer)stack.pop()).intValue();
		return(value);
    }

    @SuppressWarnings("unchecked")
	public int push(int value) {
		stack.push(new Integer(value));
		return(0);
    }

    public boolean isStackEmpty() {
		return(stack.empty());
    }

    public boolean allInputRead() {
		if(inputMachine==null) {
		  String value = inputBuffer.toString();
		  if(posToken == value.length()) return(true);
		  else return(false);
	    } else {
		  if(getLastToken()==null) return true;
		  else return false;
	    }
    }


    @SuppressWarnings("unchecked")
	public void appendFinalState(Rule rule) {
		if(rule.hasFinal()) {
		  finals.add( new Integer(rule.getFinalState()));
	    }
    }

    public boolean currentInFinals() {
		boolean res = finals.contains( new Integer(currentState) );
		return(res);
	}

    public void setMaxSpeed() {
		setSpeed(MAX_SPEED);
    }

    public void setSpeed(int speed) {
		stepSpeed = speed;
    }


    public void setOutputMachine(Vmds vmds) {
		outputMachine = vmds;
    }

    public void setInputMachine(Vmds vmds) {
		inputMachine = vmds;
    }

    public void setObjectFileName(String name) {
		objectFileName = name;
		if(!noFileName()) loadObjectCode();
    }

    public void setLastVisitedDir(File file) {
		if(file == null) currentDir = "";
		else {
		  currentDir = file.getParent();
		  if(currentDir == null) currentDir = "";
	    }
    }

    public void setLastVisitedDir(String name) {
		currentDir = (new File(name)).getParent();
		if(currentDir == null) currentDir = "";
    }

    public String getCurrentDir(){return currentDir;}

    public void loadInput() {
         inputBuffer = new StringBuffer(Utils.fileToString(currentDir,inputFileName));
    }

    private void saveOutput() {

	       FileOutputStream fos;

	       try {
	          fos = new FileOutputStream(outputFileName);

	          fos.write( (outputBuffer.toString()).getBytes());
	          fos.close();
	       }
	       catch (IOException e) {
	          System.err.println(e);
	       }

	}

    private void saveTransducerOutput() {

	       FileOutputStream fos;

	       try {
	          fos = new FileOutputStream(transducerOutputFileName);

	          fos.write( (transducerBuffer.toString()).getBytes());
	          fos.close();
	       }
	       catch (IOException e) {
	          System.err.println(e);
	       }

	}



    public void setInputFileName(String name) {
		inputFileName = name;
		loadInput();
    }

    public void setOutputFileName(String name) {
		outputFileName = name;
    }

    public void setTransducerOutputFileName(String name) {
		transducerOutputFileName = name;
    }

    public void setInputText(String text) {
        inputBuffer = new StringBuffer(text);
    }

    public void setDebugLevel(int level) {
		debugLevel = level;
		if(debugLevel > 0) System.out.println("\nEntered debug level "+debugLevel+"...");
    }


    public String getInputText() {
        return(inputBuffer.toString());
    }

    public String getObjectFileName() {
		return objectFileName;
    }

    public Vmds getInputMachine() {
		return inputMachine;
    }

    public Vmds getOutputMachine() {
		return outputMachine;
    }

    public Rules getCode() {
		return code;
    }

    public Kernel getKernel() {
		return kernel;
    }


    public int getStepSpeed() {
		return stepSpeed;
    }


    @SuppressWarnings("unused")
	private void clearCode() {
		clearObjectCode();
        initDS();
        showCurrentToken();
        setObjectFileName("");
        setSemanticAction("");
    }

    private void clearObjectCode() {
          int lin = 0;
          int col = 0;
          while(lin < MAX_OBJECT_SIZE) {
            code.setValueAt(null,lin,col);
            ++col;
            if(col>(Rules.TOTAL_COLUMNS-1)) {
			   col = 0;
			   ++lin;
			}
          }
    }

    public synchronized void kernelWillDie() {
		if(caller != null) caller.kernelWillDie();
	}


    private boolean noFileName() {
		if( objectFileName.equals("")) return true;
		else return false;
    }

    // BAD PROJECT. THIS METHOD IS REPEATED IN VIEWER.JAVA
    @SuppressWarnings({"unchecked"})
	private void loadObjectCode() {
        try {
            String headerValue="";// Save temporarily the value of the header column.
            String destValue = "";
            String value;
            String str = Utils.fileToString(currentDir,objectFileName);
            StringTokenizer tokens = new StringTokenizer(str);
            clearObjectCode();
            int lin = 0;
            int col = 0;
			finals = new Vector();
			// Get file version token.
			if(tokens.hasMoreTokens()) {
				value = tokens.nextToken();

				if(value.equals("[Version]")) {
					value = tokens.nextToken();
					if(value.equals("2")) {
                       tokens = new StringTokenizer(str.substring(12),FIELD_DELIMITER+"\n");
                    }
					else if(value.equals("1")) {
                    }
                    else {
						System.out.println("Wrong object file");
					    return;
				    }
			    } else {
					importObjectCodeV0();
					return;
			    }
		    }
            while(tokens.hasMoreTokens()) {
               value = tokens.nextToken();
               if(col==Rules.DEST_COL) {
		           destValue = value;
		       }
		       if(col==Rules.TIME_COL) {
		           headerValue = value;
			       ++col; // First column is not used !!!!!!!
		       }
               code.setValueAt(value,lin,col);
               if( !Rule.isFunction(headerValue) && value.equals("fin" ) ) {
                   String state = "";
				   if(destValue.equals("pop")) {
				      state = ((String) code.getValueAt(lin,Rules.ORIG_COL));
			       } else {
				      state = ((String) code.getValueAt(lin,Rules.DEST_COL));
			       }

				   finals.add( new Integer(state));
		       }
               ++col;
               if(col>(Rules.TOTAL_COLUMNS-1)) {
				   colorizeRule(lin,"",headerValue);
				   col = 0;
				   ++lin;
			   }
            }

        } catch (Exception e) {
            System.err.println(e.getMessage());
        }

    }

     /**
      * Used to return the machine to the original configuration
      * Currently, it simple reload the object code (MUST BE IMPROVED). 
      */
    private void restoreObjectCode() {
		loadObjectCode();
    }

    @SuppressWarnings("unchecked")
	private void importObjectCodeV0() {
        try {
//            System.out.println("Importing "+objectFileName+" ...");
            String value;
            String str = Utils.fileToString(currentDir,objectFileName);
            StringTokenizer tokens = new StringTokenizer(str);
            clearObjectCode();
            int lin = 0;
            int col = 0;
			finals = new Vector();
			// Get file version token.
            while(tokens.hasMoreTokens()) {
               value = tokens.nextToken();
               if(col==Rules.TIME_COL) {
			       ++col; // First column is not used !!!!!!!
		       }
		       if(col==Rules.HEAD_COL) { // ColorizeRule must be called after the time and header
		                           // value are read.
				   code.setValueAt("Rule",lin,col);
				   ++col;
		       }
               code.setValueAt(value,lin,col);
               if( value.equals("fin" ) ) {
				   finals.add( new Integer((String) code.getValueAt(lin,Rules.ORIG_COL)));
		       }
               ++col;
               if(col==Rules.ADAP_COL) {
				   code.setValueAt("nop",lin,col);
				   ++col; // Adap col not used.
		       }
               if(col>(Rules.TOTAL_COLUMNS-1)) {
				   colorizeRule(lin,"","Rule");
				   col = 0;
				   ++lin;
			   }
            }

        } catch (Exception e) {
            System.err.println(e.getMessage());
        }

    }

    // UGLY. Must be improved. Too many parameters.
    public void initConnection(String dir, String fileName, int posX, int posY, Vmds outputMachine, String inputFileName, String semanticAction ) {
            currentDir = dir;
            init(fileName);
            setOutputMachine(outputMachine);
            setInputFileName(inputFileName);
            loadInput();
            setObjectFileName(fileName);
    }


    public void connectInput(String dir,String fileName) {
            setInputMachine(new VmdsImpl());
            String inputFileNameTemp;
//            if(inputFileName.length()>0 && inputFileName.charAt(0) != '/')
 //               inputFileNameTemp = currentDir + "/" + inputFileName;
  //          else
                inputFileNameTemp = inputFileName;
                System.out.println("ileName = "+fileName);
                System.out.println("inputFileName = "+inputFileName);
                System.out.println("inputFileNameTemp = "+inputFileNameTemp);
            inputMachine.initConnection(dir,fileName,0,0,this,inputFileNameTemp, null);
    }


    private boolean noTransducerOutputFileName() {
		if( transducerOutputFileName.equals("")) return true;
		else return false;
    }


    public void executionFinished() {
		if( outputFileName != null ) {
           saveOutput();
           if(!noTransducerOutputFileName()) saveTransducerOutput();
	    }
	    if(outputMachine == null) {
//			if(debugLevel > 0) System.out.println(transducerBuffer.toString());
			System.out.print("Finished\n");
//			System.exit(0);
	    }

    }

    /**
     * Run on imput saved on file
     * @param vmds
     */
    public void run(Vmds pcaller) {
		caller = pcaller;
		if(inputMachine != null) {
			setInputText("");
			inputMachine.run(null);
		}
        restoreObjectCode();

		kernel = new Kernel(this);
    }



    // SOMETHING IS WRONG. I SHOULDN'T HAVE TO DECLARE ALL THIS "EMPTY" METHODS.
    // MUST BE IMPROVED LATER

    /** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
 	public boolean noInputFileName(){return true;}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
 	public void saveInput(){}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
    public String getInputFileName(){return null;}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
	public boolean isAnimate() {return false;}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
	public boolean isAnimateMulti() {return false;}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
	public boolean isAnimateSingle() {return false;}
 	
	/** this mtodo is olny used on the graphic implementation
 	 * I think so that, this is can be in other interface
 	 *  (Denys G Santos) 
 	 */
    public void clearWatch() {}
 	
    /** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
    public void setWatch(Hashtable bindings) {}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
	public ObjectCodeTable getCodeTable() {return null;}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
    public void colorizeAllRules(){}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
    public boolean isRemoveDefinitely() {return false;}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
    public void removeDefinitely() {}
    /** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
    public void setRunning(boolean value) {}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
    public void setSleeping(boolean value) {}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
    public void colorizeRule(int line, String timeValue, String headerValue) {}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
    public void showCurrentToken() {}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
    public void unsetRule(Rule rule) {}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
    public void setRule(Rule rule) {}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
    public boolean isRuleSet(Rule rule) {return true;}
 
    public void unsetAction(Rule rule) {}

    public void setAction(Rule Action) {}
    
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
    public void unsetQueryResult(Rule rule) {}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
    public void setQueryResult(Rule rule) {}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
    public void suicide() {}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
    public void reset() {}
 	/** Este mtodo no faz nada... nem devia existir (Denys G Santos) */
    public void saveAndExit() {}

    /**
     * return the Semantic Actions
     */
	public String getSemanticAction() {
		return semanticAction;
	}

	/**
	 * set the semantics actions
	 * @param semanticAction
	 */
	public void setSemanticAction(String semanticAction) {
		this.semanticAction = semanticAction;
	}


}
