/** Semantics.java
 
 Provides the semantics for a pushdown state automata that implements
 a compiler-compiler invented by Dr. Joao Jose Neto. The input grammar
 must be in the Wirth notation.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 @author Hemerson Pistori (pistori@ec.ucdb.br)
 
 This work is based on Dr. Joao Jose Neto theories and techniques on
 Automata and Formal Languages (http://www.pcs.usp.br/~lta).
 
 
*/

package cc;

import java.util.Hashtable;
import java.util.Stack;

import adaptools.vm.Token;
import adaptools.vm.Vmds;


public class Semantics implements adaptools.vm.Semantics {

    Stack stack;
    Hashtable subMacs;  // Address of sub-submachines;
    Hashtable segNames;  // Address of sub-submachines;
    int macCounter;
    int segSize = 1000;  // Size of a sub-machine segment.
    int counter;
    int curState;

    public Vmds vmds;
    
    public void init(Vmds vmds) {
        this.vmds = vmds;
        stack = new Stack();
    }
    
    public Token execute(String name) {
        if(name.equals("rs0"))
            return(rs0());
        else if(name.equals("rs1"))
            return(rs1());
        else if(name.equals("rs2"))
            return(rs2());
        else if(name.equals("rs3"))
            return(rs3());
        else if(name.equals("rs4"))
            return(rs4());
        else if(name.equals("rs5"))
            return(rs5());
        else if(name.equals("rs6"))
            return(rs6());
        else if(name.equals("rs7"))
            return(rs7());
        else if(name.equals("rs8"))
            return(rs8());
        else
            return(null);
    }

    // No terminal do lado esquerdo da produo
    private Token rs0() {

        stack.clear();

        macCounter = 0;

        subMacs = new Hashtable();

        segNames = new Hashtable();

        rs7();

        return new Token("[Version] 2\n","");
    }


    // No terminal (lado direito),  terminal ou cadeia vazia
    private Token rs1() {
        StringBuffer buffer = new StringBuffer();

        buffer.append(getCurMachine()+vmds.FIELD_DELIMITER);
        buffer.append(getCurState()+vmds.FIELD_DELIMITER);

        if( vmds.curToken().label.equals("&"))
            buffer.append("eps"+vmds.FIELD_DELIMITER);
        else if( vmds.curToken().label.equals("t"))
            buffer.append(vmds.curToken().value+vmds.FIELD_DELIMITER);
        else if( vmds.curToken().label.equals("n"))
            buffer.append("eps"+vmds.FIELD_DELIMITER);

        if( vmds.curToken().label.equals("n"))
            buffer.append(getSubMacAddress()+vmds.FIELD_DELIMITER);
        else
            buffer.append(getCounter()+vmds.FIELD_DELIMITER);

        if( vmds.curToken().label.equals("n"))
            buffer.append(getCounter()+vmds.FIELD_DELIMITER);
        else
            buffer.append("nop"+vmds.FIELD_DELIMITER);

        buffer.append("nop"+vmds.FIELD_DELIMITER);
        buffer.append("nop\n");

        setCurState(counter);
        incCounter();

        return new Token(buffer.toString(),"");
    }


    // Abertura de parnteses ou sinal de igualdade
    private Token rs2() {

        String [] pair = new String[2];

        pair[0] = getCurState();
        pair[1] = getCounter();

        stack.push(pair);

        incCounter();

        return null;
    }

    // Abertura de colchetes
    private Token rs3() {
        StringBuffer buffer = new StringBuffer();

        String [] pair = new String[2];

        buffer.append(getCurMachine()+vmds.FIELD_DELIMITER);
        buffer.append(getCurState()+vmds.FIELD_DELIMITER);
        buffer.append("eps"+vmds.FIELD_DELIMITER);
        buffer.append(getCounter()+vmds.FIELD_DELIMITER);
        buffer.append("nop"+vmds.FIELD_DELIMITER);
        buffer.append("nop"+vmds.FIELD_DELIMITER);
        buffer.append("nop\n");

        pair[0] = getCurState();
        pair[1] = getCounter();
        stack.push(pair);

        incCounter();


        return new Token(buffer.toString(),"");
    }

    // Abertura de chaves
    private Token rs4() {
        StringBuffer buffer = new StringBuffer();

        String [] pair = new String[2];

        buffer.append(getCurMachine()+vmds.FIELD_DELIMITER);
        buffer.append(getCurState()+vmds.FIELD_DELIMITER);
        buffer.append("eps"+vmds.FIELD_DELIMITER);
        buffer.append(getCounter()+vmds.FIELD_DELIMITER);
        buffer.append("nop"+vmds.FIELD_DELIMITER);
        buffer.append("nop"+vmds.FIELD_DELIMITER);
        buffer.append("nop\n");

        setCurState(counter);

        pair[0] = getCounter();
        pair[1] = getCounter();
        stack.push(pair);

        incCounter();

        return new Token(buffer.toString(),"");
    }

    // Fechamento de parenteses, chaves, colchetes
    private Token rs5() {
        StringBuffer buffer = new StringBuffer();

        String [] pair = (String []) stack.peek();

        buffer.append(getCurMachine()+vmds.FIELD_DELIMITER);
        buffer.append(getCurState()+vmds.FIELD_DELIMITER);
        buffer.append("eps"+vmds.FIELD_DELIMITER);
        buffer.append(pair[1]+vmds.FIELD_DELIMITER);
        buffer.append("nop"+vmds.FIELD_DELIMITER);
        buffer.append("nop"+vmds.FIELD_DELIMITER);
        buffer.append("nop\n");

        setCurState(pair[1]);
        stack.pop();

        return new Token(buffer.toString(),"");
    }

    // Barra Vertical
    private Token rs6() {
        StringBuffer buffer = new StringBuffer();

        String [] pair = (String []) stack.peek();

        buffer.append(getCurMachine()+vmds.FIELD_DELIMITER);
        buffer.append(getCurState()+vmds.FIELD_DELIMITER);
        buffer.append("eps"+vmds.FIELD_DELIMITER);
        buffer.append(pair[1]+vmds.FIELD_DELIMITER);
        buffer.append("nop"+vmds.FIELD_DELIMITER);
        buffer.append("nop"+vmds.FIELD_DELIMITER);
        buffer.append("nop\n");

        setCurState(pair[0]);


        return new Token(buffer.toString(),"");
    }

    // No terminal do lado esquerdo da produo
    private Token rs7() {

        String [] pair = new String[2];


        setCurState(getSubMacAddress());
        counter = curState+1;

        pair[0] = getCurState();
        pair[1] = getCounter();
        stack.push(pair);

        return null;
    }


    // Final da producao (.)
    private Token rs8() {
        StringBuffer buffer = new StringBuffer();

        buffer.append(rs5().label);

        buffer.append(getCurMachine()+vmds.FIELD_DELIMITER);
        buffer.append(getCurState()+vmds.FIELD_DELIMITER);
        buffer.append("eps"+vmds.FIELD_DELIMITER);
        buffer.append("pop"+vmds.FIELD_DELIMITER);
        buffer.append("fin"+vmds.FIELD_DELIMITER);
        buffer.append("nop"+vmds.FIELD_DELIMITER);
        buffer.append("nop\n");

        return new Token(buffer.toString(),"");
    }



    private void setCurState(int value) {
        curState = value;
    }

    private void setCurState(String value) {
        curState = Integer.parseInt(value.trim());
    }


    private String getSubMacAddress() {
        Integer address;
        address = (Integer) subMacs.get(vmds.curToken().value);


        if( address == null ) {
            subMacs.put(vmds.curToken().value,new Integer(macCounter*segSize));

            segNames.put(new Integer(macCounter),vmds.curToken().value);
            ++macCounter;
            return( (new Integer((macCounter-1)*segSize)).toString() );
        } else {
            return(  address.toString() );
        }
    }

    String getCurState() {
        return  (new Integer(curState)).toString() ;
    }

    String getCurMachine() {

        int machine = (new Integer(curState)).intValue() / 1000;


        String name = (String) segNames.get(new Integer(machine));


        return( name );

    }


    private void incCounter() {
        ++counter;
    }

    String getCounter() {
        return( (new Integer(counter)).toString() );
    }

}
