/** Semantics.java
 
 Provides the semantics for a pushdown state automata that implements
 a compiler-compiler invented by Dr. Joao Jose Neto. The input grammar
 must be in the Wirth notation.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 @author Hemerson Pistori (pistori@ec.ucdb.br)
 
 This work is based on Dr. Joao Jose Neto theories and techniques on
 Automata and Formal Languages (http://www.pcs.usp.br/~lta).
 
 
*/

package ccl;

import adaptools.vm.Token;
import adaptools.vm.Vmds;


public class Semantics implements adaptools.vm.Semantics  {

    StringBuffer value;

    public Vmds vmds;

    public void init(Vmds vmds) {
        this.vmds = vmds;
        value = new StringBuffer();
    }

    public Token execute(String name) {
        if(name.equals("_initBuffer"))
            return(_initBuffer());
        else if(name.equals("_getBuffer"))
            return(_getBuffer());
        else if(name.equals("_appendToBuffer"))
            return(_appendToBuffer());
        else if(name.equals("_terminal"))
            return(_terminal());
        else if(name.equals("_nonTerminal"))
            return(_nonTerminal());
        else if(name.equals("_metaSymbol"))
            return(_metaSymbol());
        else if(name.equals("_bufferToken"))
            return(_bufferToken());
        else if(name.startsWith("_token"))
            return(_token(name));
        else
            return(null);
    }



    private Token _initBuffer() {
        value = new StringBuffer();
        return null;
    }

    private Token _getBuffer() {
        return vmds.curToken();
    }

    private Token _appendToBuffer() {
        value.append(vmds.curToken().value);
        return null;
    }

    private Token _terminal() {
        return new Token("t",value.toString());
    }

    private Token _nonTerminal() {
        return new Token("n",value.toString());
    }

    private Token _metaSymbol() {
        //System.out.println("Returning metaSymbol = ["+vmds.curToken().value+"]");
        return new Token(vmds.curToken().value,vmds.curToken().value);
    }

    private Token _bufferToken() {
        String temp = value.toString();
        _initBuffer();
        //System.out.println("Returning buffer token with ["+temp+"]");
        return new Token(temp,temp);
    }

    private Token _token(String temp) {
        temp = temp.substring(temp.indexOf('(')+1,temp.indexOf(')'));
        //System.out.println("Returning parameter token with ["+temp+"]");
        return new Token(temp,temp);
    }


}
