/** Semantics.java
 
 Provides the semantics for a pushdown state automata that implements
 a compiler-compiler invented by Dr. Joao Jose Neto. The input grammar
 must be in the Wirth notation.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 @author Hemerson Pistori (pistori@ec.ucdb.br)
 
 This work is based on Dr. Joao Jose Neto theories and techniques on
 Automata and Formal Languages (http://www.pcs.usp.br/~lta).
 
 
*/

package ccr;

import java.util.Hashtable;
import java.util.Stack;

import adaptools.vm.Token;
import adaptools.vm.Vmds;



public class Semantics implements adaptools.vm.Semantics {

    Stack stack;
    Hashtable subMacs;  // Address of sub-submachines;
    int macCounter;
    int segSize = 1000;  // Size of a sub-machine segment.
    int counter;
    int curState;
    int columnSize;

    public Vmds vmds;

    public void init(Vmds vmds) {
        this.vmds = vmds;
        columnSize = 7;
        stack = new Stack();
    }


    public Token execute(String name) {
        if(name.equals("rs0"))
            return(rs0());
        else if(name.equals("rs1"))
            return(rs1());
        else if(name.equals("rs2"))
            return(rs2());
        else if(name.equals("rs3"))
            return(rs3());
        else if(name.equals("rs4"))
            return(rs4());
        else if(name.equals("rs5"))
            return(rs5());
        else if(name.equals("rs6"))
            return(rs6());
        else if(name.equals("rs7"))
            return(rs7(false));
        else if(name.equals("rs8"))
            return(rs8());
        else
            return(null);
    }

    // No terminal do lado esquerdo da produo
    private Token rs0() {

        StringBuffer adaptiveFunction;

        stack.clear();

        macCounter = 0;

        subMacs = new Hashtable();

        adaptiveFunction = creatErrorRecoveryAdaptiveFunction();

        rs7(true);

        return new Token("[Version] 1\n"+adaptiveFunction,"");
    }


    // No terminal (lado direito),  terminal ou cadeia vazia
    private Token rs1() {
        StringBuffer buffer = new StringBuffer();

        buffer.append(getCurMachine());
        buffer.append(getCurState());

        if( vmds.curToken().label.equals("&"))
            buffer.append(formatN("eps",11));
        else if( vmds.curToken().label.equals("t"))
            buffer.append(formatN(vmds.curToken().value,11));
        else if( vmds.curToken().label.equals("n"))
            buffer.append(formatN("eps",11));

        if( vmds.curToken().label.equals("n"))
            buffer.append(getSubMacAddress());
        else
            buffer.append(getCounter());

        if( vmds.curToken().label.equals("n"))
            buffer.append(getCounter());
        else
            buffer.append(format("nop"));

        buffer.append("nop ");
        buffer.append("E2.E1\n");

        setCurState(counter);
        incCounter();

        return new Token(buffer.toString(),"");
    }


    // Abertura de parnteses ou sinal de igualdade
    private Token rs2() {

        String [] pair = new String[2];

        pair[0] = getCurState();
        pair[1] = getCounter();

        stack.push(pair);

        incCounter();

        return null;
    }

    // Abertura de colchetes
    private Token rs3() {
        StringBuffer buffer = new StringBuffer();

        String [] pair = new String[2];

        buffer.append(getCurMachine());
        buffer.append(getCurState());
        buffer.append(formatN("eps",11));
        buffer.append(getCounter());
        buffer.append(format("nop"));
        buffer.append("nop ");
        buffer.append("E2.E1\n");

        pair[0] = getCurState();
        pair[1] = getCounter();
        stack.push(pair);

        incCounter();


        return new Token(buffer.toString(),"");
    }

    // Abertura de chaves
    private Token rs4() {
        StringBuffer buffer = new StringBuffer();

        String [] pair = new String[2];

        buffer.append(getCurMachine());
        buffer.append(getCurState());
        buffer.append(formatN("eps",11));
        buffer.append(getCounter());
        buffer.append(format("nop"));
        buffer.append("nop ");
        buffer.append("E2.E1\n");

        setCurState(counter);

        pair[0] = getCounter();
        pair[1] = getCounter();
        stack.push(pair);

        incCounter();

        return new Token(buffer.toString(),"");
    }

    // Fechamento de parenteses, chaves, colchetes
    private Token rs5() {
        StringBuffer buffer = new StringBuffer();

        String [] pair = (String []) stack.peek();

        buffer.append(getCurMachine());
        buffer.append(getCurState());
        buffer.append(formatN("eps",11));
        buffer.append(pair[1]);
        buffer.append(format("nop"));
        buffer.append("nop ");
        buffer.append("E2.E1\n");

        setCurState(pair[1]);
        stack.pop();

        return new Token(buffer.toString(),"");
    }

    // Barra Vertical
    private Token rs6() {
        StringBuffer buffer = new StringBuffer();

        String [] pair = (String []) stack.peek();

        buffer.append(getCurMachine());
        buffer.append(getCurState());
        buffer.append(formatN("eps",11));
        buffer.append(pair[1]);
        buffer.append(format("nop"));
        buffer.append("nop ");
        buffer.append("E2.E1\n");

        setCurState(pair[0]);


        return new Token(buffer.toString(),"");
    }

    // No terminal do lado esquerdo da produo
    private Token rs7(boolean firstSubMachine) {

        String [] pair = new String[2];

        setCurState(getSubMacAddress());
        counter = curState+1;

        if(firstSubMachine) {
            setCurState(1);
            counter = curState+1;
        }

        pair[0] = getCurState();
        pair[1] = getCounter();
        stack.push(pair);

        return null;
    }


    // Final da producao (.)
    private Token rs8() {
        StringBuffer buffer = new StringBuffer();

        buffer.append(rs5().label);

        buffer.append(getCurMachine());
        buffer.append(getCurState());
        buffer.append(formatN("eps",11));
        buffer.append(format("pop"));
        buffer.append(format("fin"));
        buffer.append("nop ");
        buffer.append("E2.E1\n");

        return new Token(buffer.toString(),"");
    }


    private StringBuffer creatErrorRecoveryAdaptiveFunction() {

        StringBuffer buffer = new StringBuffer();

        buffer.append("?E1  ?sta   ?first   ?p     ?f1    ?f2    ?f3\n");
        buffer.append("?E1  ?p     ?second  ?q     ?s1    ?s2    ?s3\n");
        buffer.append("+E1  ?sta   ?second  *rem   ?s1    ?s2    ERR\n");
        buffer.append("+E1  *rem   eps      ?q     nop    <R>    ERR\n");
        buffer.append("+E1  ?sta   eps      *n1    ?f1    nop    ERR\n");
        buffer.append("+E1  *n1    a        *n2    nop    [A]    ERR\n");
        buffer.append("+E1  *n1    b        *n2    nop    [B]    ERR\n");
        buffer.append("+E1  *n1    c        *n2    nop    [C]    ERR\n");
        buffer.append("+E1  *n2    ?first   *ins   ?f1    ?f2    ERR\n");
        buffer.append("+E1  *ins   eps      ?p     nop    <I>    ERR\n");
        buffer.append("+E1  *n2    ?second  *sub   ?s1    ?s2    ERR\n");
        buffer.append("+E1  *sub   eps      ?q     nop    <S>    ERR\n");
        buffer.append("?E2  ?z1    ?z2      ?z3    ?z4    ?z5    ERR\n");
        buffer.append("-E2  ?z1    ?z2      ?z3    ?z4    ?z5    ERR\n");

        // Empty Transition before first state.
        buffer.append("M0   0      eps        1      nop    nop E2.E1\n");

        return(buffer);
    }


    private void setCurState(int value) {
        curState = value;
    }

    private void setCurState(String value) {
        curState = Integer.parseInt(value.trim());
    }


    private String getSubMacAddress() {
        Integer address;
        address = (Integer) subMacs.get(vmds.curToken().value);


        if( address == null ) {
            subMacs.put(vmds.curToken().value,new Integer(macCounter*segSize));
            ++macCounter;
            return( format((macCounter-1)*segSize) );
        } else {
            return( format( address.toString() ));
        }
    }

    String getCurState() {
        return( format( (new Integer(curState)).toString() ));
    }

    String getCurMachine() {

        int machine = (new Integer(curState)).intValue() / 1000;

        return( "M"+formatN( (new Integer(machine)).toString(),4));
    }


    private void incCounter() {
        ++counter;
    }

    String getCounter() {
        return( format(  (new Integer(counter)).toString() ));
    }

    private String format(String str) {
        String nstr = str;
        for(int i = nstr.length() ; i < columnSize; ++i )
            nstr = nstr + " ";
        return(nstr);
    }

    private String format(int value) {
        String nstr = new Integer(value).toString();
        for(int i = nstr.length() ; i < columnSize; ++i )
            nstr = nstr + " ";
        return(nstr);
    }

    // Add spaces until the n length is achieved
    private String formatN(String str,int n) {
        String nstr = str;
        for(int i = nstr.length() ; i < n; ++i )
            nstr = nstr + " ";
        return(nstr);
    }

    // Prefix the string with 0 to achieve an exact length of n-1
    // and then sufix the string with 1 space.
    private String formatZN(String str,int n) {

        String nstr = str;
        for(int i = nstr.length() ; i < (n-1); ++i )
            nstr = "0" + nstr;
        return(nstr+" ");
    }
}
